#if !defined(AFX_RESIZABLEDIALOG_H__21FE3CD1_979B_11D3_A80F_525400E146A5__INCLUDED_)
#define AFX_RESIZABLEDIALOG_H__21FE3CD1_979B_11D3_A80F_525400E146A5__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// ResizableDialog.h : header file

 
//===========================================================================
// Summary:
//     The CResizableDialog class derived from CDialog
//      Resizable Dialog
//===========================================================================

class CResizableDialog : public CDialog
{

// Construction
public:
	
	//-----------------------------------------------------------------------
	// Summary:
	// Resizable Dialog, Constructs a CResizableDialog object.
	//		Returns A  value.
	CResizableDialog();
	
	//-----------------------------------------------------------------------
	// Summary:
	// Resizable Dialog, Constructs a CResizableDialog object.
	//		Returns A  value.  
	// Parameters:
	//		nIDTemplate---I D Template, Specifies A 16-bit unsigned integer on Windows versions 3.0 and 3.1; a 32-bit unsigned integer on Win32.  
	//		pParentWnd---Parent Window, A pointer to the CWnd or NULL if the call failed.
	CResizableDialog(UINT nIDTemplate, CWnd* pParentWnd = NULL);

// Attributes
public:
	enum AnchorType
	{
		NOANCHOR,
		TOP_LEFT,		TOP_CENTER,		TOP_RIGHT,
		MIDDLE_LEFT,	MIDDLE_CENTER,	MIDDLE_RIGHT,
		BOTTOM_LEFT,	BOTTOM_CENTER,	BOTTOM_RIGHT
	};

private:
	// flags
 
	// Show Grip, This member sets TRUE if it is right.  
	BOOL m_bShowGrip;
 
	// Use Maximize Track, This member sets TRUE if it is right.  
	BOOL m_bUseMaxTrack;
 
	// Use Minimize Track, This member sets TRUE if it is right.  
	BOOL m_bUseMinTrack;
 
	// Use Maximize Rectangle, This member sets TRUE if it is right.  
	BOOL m_bUseMaxRect;
 
	// Enable Save Restore, This member sets TRUE if it is right.  
	BOOL m_bEnableSaveRestore;

	// internal status
 
	// m_sSection, You can freely substitute CString objects for const char* and LPCTSTR function arguments.  
	CString m_sSection;			// section name and
 
	// m_sEntry, You can freely substitute CString objects for const char* and LPCTSTR function arguments.  
	CString m_sEntry;			// entry for save/restore

 
	// Initial Done, This member sets TRUE if it is right.  
	BOOL m_bInitDone;			// if all internal vars initialized

 
	// Grip Size, This member sets a CSize value.  
	CSize m_szGripSize;			// set at construction time

 
	// Grip Rectangle, This member sets a CRect value.  
	CRect m_rcGripRect;			// current pos of grip

 
	// Minimize Track Size, This member specify POINT object.  
	POINT m_ptMinTrackSize;		// min tracking size
 
	// Maximize Track Size, This member specify POINT object.  
	POINT m_ptMaxTrackSize;		// max tracking size
 
	// Maximize Position, This member specify POINT object.  
	POINT m_ptMaxPos;			// maximized position
 
	// Maximize Size, This member specify POINT object.  
	POINT m_ptMaxSize;			// maximized size

 
	// Layout List, This member specify CPtrList object.  
	CPtrList m_plLayoutList;	// list of repositionable controls

	class Layout
	{
	public:
		HWND hwnd;

		BOOL adj_hscroll;

		// upper-left corner
		AnchorType ul_type;
		SIZE ul_margin;
		
		// bottom-right corner
		AnchorType br_type;
		SIZE br_margin;
	
	public:
		Layout(HWND hw, AnchorType ul_t, SIZE ul_m, 
			AnchorType br_t, SIZE br_m, BOOL hscroll)
		{
			hwnd = hw;

			adj_hscroll = hscroll;

			ul_type = ul_t;
			ul_margin = ul_m;
			
			br_type = br_t;
			br_margin = br_m;
		};
	};

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CResizableDialog)
	//}}AFX_VIRTUAL

// Implementation
public:
	
	//-----------------------------------------------------------------------
	// Summary:
	// C Resizable Dialog, Destructor of class CResizableDialog
	// This member function is also a virtual function, you can Override it if you need,  
	//		Returns A  value.
	virtual ~CResizableDialog();

// used internally
private:
	
	//-----------------------------------------------------------------------
	// Summary:
	// Load Window Rectangle, Call this function to read a specified number of bytes from the archive.

	void LoadWindowRect();
	
	//-----------------------------------------------------------------------
	// Summary:
	// Save Window Rectangle, Call this function to save the specify data to a file.

	void SaveWindowRect();
	
	//-----------------------------------------------------------------------
	// Summary:
	// Arrange Layout, None Description.

	void ArrangeLayout();
	
	//-----------------------------------------------------------------------
	// Summary:
	// Update Grip Position, Call this member function to update the object.

	void UpdateGripPos();

// callable from derived classes
protected:
	
	//-----------------------------------------------------------------------
	// Summary:
	// Add Anchor, Adds an object to the specify list.
	//		Returns TRUE on success; FALSE otherwise.  
	// Parameters:
	//		pWnd---pWnd, A pointer to the CWnd or NULL if the call failed.  
	//		ul_type---Specifies a AnchorType ul_type object.  
	//		br_type---Specifies a AnchorType br_type = NOANCHOR object.
	BOOL AddAnchor(CWnd* pWnd, AnchorType ul_type,
		AnchorType br_type = NOANCHOR);	// add anchors to a control
	
	//-----------------------------------------------------------------------
	// Summary:
	// Add Anchor, Adds an object to the specify list.
	//		Returns TRUE on success; FALSE otherwise.  
	// Parameters:
	//		ctrl_ID---I D, Specifies A 16-bit unsigned integer on Windows versions 3.0 and 3.1; a 32-bit unsigned integer on Win32.  
	//		ul_type---Specifies a AnchorType ul_type object.  
	//		br_type---Specifies a AnchorType br_type = NOANCHOR object.
	BOOL AddAnchor(UINT ctrl_ID, AnchorType ul_type,
		AnchorType br_type = NOANCHOR)	// add anchors to a control
	{
		return AddAnchor(GetDlgItem(ctrl_ID), ul_type, br_type);
	};
	
	//-----------------------------------------------------------------------
	// Summary:
	// Show Size Grip, Call this function to show the specify object.
	// Parameters:
	//		bShow---bShow, Specifies A Boolean value.
	void ShowSizeGrip(BOOL bShow);				// show or hide the size grip
	
	//-----------------------------------------------------------------------
	// Summary:
	// Set Maximized Rectangle, Sets a specify value to current class CResizableDialog
	// Parameters:
	//		rc---Specifies A CRect type value.
	void SetMaximizedRect(const CRect& rc);		// set window rect when maximized
	
	//-----------------------------------------------------------------------
	// Summary:
	// Reset Maximized Rectangle, Called this function to empty a previously initialized CResizableDialog object.

	void ResetMaximizedRect();					// reset to default maximized rect
	
	//-----------------------------------------------------------------------
	// Summary:
	// Set Minimize Track Size, Sets a specify value to current class CResizableDialog
	// Parameters:
	//		size---Specifies A CSize type value.
	void SetMinTrackSize(const CSize& size);	// set minimum tracking size
	
	//-----------------------------------------------------------------------
	// Summary:
	// Reset Minimize Track Size, Called this function to empty a previously initialized CResizableDialog object.

	void ResetMinTrackSize();					// reset to default minimum tracking size
	
	//-----------------------------------------------------------------------
	// Summary:
	// Set Maximize Track Size, Sets a specify value to current class CResizableDialog
	// Parameters:
	//		size---Specifies A CSize type value.
	void SetMaxTrackSize(const CSize& size);	// set maximum tracking size
	
	//-----------------------------------------------------------------------
	// Summary:
	// Reset Maximize Track Size, Called this function to empty a previously initialized CResizableDialog object.

	void ResetMaxTrackSize();					// reset to default maximum tracking size
	
	//-----------------------------------------------------------------------
	// Summary:
	// Enable Save Restore, Call this member function to enable or disable the specify object for this command.
	// Parameters:
	//		pszSection---pszSection, Specifies A 32-bit pointer to a constant character string that is portable for Unicode and DBCS.  
	//		pszEntry---pszEntry, Specifies A 32-bit pointer to a constant character string that is portable for Unicode and DBCS.
	void EnableSaveRestore(LPCTSTR pszSection, LPCTSTR pszEntry);	// section and entry in app's profile

// Generated message map functions
protected:
	//{{AFX_MSG(CResizableDialog)
	
	//-----------------------------------------------------------------------
	// Summary:
	// On Initial Dialog, This member function is called by the framework to allow your application to handle a Windows message.
	// This member function is also a virtual function, you can Override it if you need,  
	//		Returns TRUE on success; FALSE otherwise.
	virtual BOOL OnInitDialog();
	
	//-----------------------------------------------------------------------
	// Summary:
	// On Paint, Called to repaint a portion of the window.

	afx_msg void OnPaint();
	
	//-----------------------------------------------------------------------
	// Summary:
	// On Nc Hit Test, This member function is called by the framework to allow your application to handle a Windows message.
	//		Returns A afx_msg FOPNcHitTestType value.  
	// Parameters:
	//		point---Specifies A CPoint type value.
	afx_msg FOPNcHitTestType OnNcHitTest(CPoint point);
	
	//-----------------------------------------------------------------------
	// Summary:
	// On Get Minimize Maximize Information, This member function is called by the framework to allow your application to handle a Windows message.
	// Parameters:
	//		lpMMI---M M I, A pointer to the MINMAXINFO FAR or NULL if the call failed.
	afx_msg void OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI);
	
	//-----------------------------------------------------------------------
	// Summary:
	// On Size, Called after the size of CWnd has changed.
	// Parameters:
	//		nType---nType, Specifies A 16-bit unsigned integer on Windows versions 3.0 and 3.1; a 32-bit unsigned integer on Win32.  
	//		cx---Specifies A integer value.  
	//		cy---Specifies A integer value.
	afx_msg void OnSize(UINT nType, int cx, int cy);
	
	//-----------------------------------------------------------------------
	// Summary:
	// On Destroy, Called when CWnd is being destroyed.

	afx_msg void OnDestroy();
	//}}AFX_MSG
	
	//-----------------------------------------------------------------------
	// Summary:
	// E C L A R E_ M E S S A G E_ M A P, None Description.
	//		Returns A  value.
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_RESIZABLEDIALOG_H__21FE3CD1_979B_11D3_A80F_525400E146A5__INCLUDED_)
